#pragma once
#include <map>
static const std::map<std::string, std::string> SHADERS = {
{"CM.glsl",
#include "./CM.glsl.inc"
},
{"CMblurprepare.frag",
#include "./CMblurprepare.frag.inc"
},
{"CMborder.frag",
#include "./CMborder.frag.inc"
},
{"blur1.frag",
#include "./blur1.frag.inc"
},
{"blur2.frag",
#include "./blur2.frag.inc"
},
{"blurfinish.frag",
#include "./blurfinish.frag.inc"
},
{"blurprepare.frag",
#include "./blurprepare.frag.inc"
},
{"border.frag",
#include "./border.frag.inc"
},
{"border.glsl",
#include "./border.glsl.inc"
},
{"discard.glsl",
#include "./discard.glsl.inc"
},
{"do_CM.glsl",
#include "./do_CM.glsl.inc"
},
{"do_discard.glsl",
#include "./do_discard.glsl.inc"
},
{"do_rounding.glsl",
#include "./do_rounding.glsl.inc"
},
{"do_sdr_mod.glsl",
#include "./do_sdr_mod.glsl.inc"
},
{"do_tint.glsl",
#include "./do_tint.glsl.inc"
},
{"do_tonemap.glsl",
#include "./do_tonemap.glsl.inc"
},
{"ext.frag",
#include "./ext.frag.inc"
},
{"gain.glsl",
#include "./gain.glsl.inc"
},
{"get_rgb_pixel.glsl",
#include "./get_rgb_pixel.glsl.inc"
},
{"get_rgba_pixel.glsl",
#include "./get_rgba_pixel.glsl.inc"
},
{"get_rgbx_pixel.glsl",
#include "./get_rgbx_pixel.glsl.inc"
},
{"glitch.frag",
#include "./glitch.frag.inc"
},
{"passthru.frag",
#include "./passthru.frag.inc"
},
{"primaries_xyz.glsl",
#include "./primaries_xyz.glsl.inc"
},
{"primaries_xyz_const.glsl",
#include "./primaries_xyz_const.glsl.inc"
},
{"primaries_xyz_uniform.glsl",
#include "./primaries_xyz_uniform.glsl.inc"
},
{"quad.frag",
#include "./quad.frag.inc"
},
{"rgbamatte.frag",
#include "./rgbamatte.frag.inc"
},
{"rounding.glsl",
#include "./rounding.glsl.inc"
},
{"sdr_mod.glsl",
#include "./sdr_mod.glsl.inc"
},
{"shadow.frag",
#include "./shadow.frag.inc"
},
{"surface.frag",
#include "./surface.frag.inc"
},
{"surface_CM.glsl",
#include "./surface_CM.glsl.inc"
},
{"tex300.vert",
#include "./tex300.vert.inc"
},
{"tex320.vert",
#include "./tex320.vert.inc"
},
{"tint.glsl",
#include "./tint.glsl.inc"
},
{"tonemap.glsl",
#include "./tonemap.glsl.inc"
},
};
