#pragma once

#include <cstdint>

enum eTrackpadGestureDirection : uint8_t {
    TRACKPAD_GESTURE_DIR_NONE = 0,
    TRACKPAD_GESTURE_DIR_SWIPE,
    TRACKPAD_GESTURE_DIR_LEFT,
    TRACKPAD_GESTURE_DIR_RIGHT,
    TRACKPAD_GESTURE_DIR_UP,
    TRACKPAD_GESTURE_DIR_DOWN,
    TRACKPAD_GESTURE_DIR_VERTICAL,
    TRACKPAD_GESTURE_DIR_HORIZONTAL,
    TRACKPAD_GESTURE_DIR_PINCH,
    TRACKPAD_GESTURE_DIR_PINCH_OUT,
    TRACKPAD_GESTURE_DIR_PINCH_IN,
};