#pragma once
#define GIT_COMMIT_HASH    "521ece463c4a9d3d128670688a34756805a4328f"
#define GIT_BRANCH         "v0.54.3"
#define GIT_COMMIT_MESSAGE "version: bump to 0.54.3"
#define GIT_COMMIT_DATE    "Fri Mar 27 18:17:50 2026"
#define GIT_DIRTY          "clean"
#define GIT_TAG            "v0.54.3"
#define GIT_COMMITS        "7014"

#define AQUAMARINE_VERSION "0.11.0"
// clang-format off
#define AQUAMARINE_VERSION_MAJOR 0
#define AQUAMARINE_VERSION_MINOR 11
#define AQUAMARINE_VERSION_PATCH 0
// clang-format on
#define HYPRLANG_VERSION     "0.6.8"
#define HYPRUTILS_VERSION    "0.12.0"
#define HYPRCURSOR_VERSION   "0.1.13"
#define HYPRGRAPHICS_VERSION "0.5.1"
